<?php

namespace UNIFIED_SOFTWARE\EMAILVAL;

const MAIN_SERVER = 'https://www.unifiedservices.co.uk';
const BACKUP_SERVER = 'https://www.unifiedsoftware.co.uk';

const BANKVAL_ENDPOINT = 'services/enhanced/bankvalnew';
const VERIFY_ENDPOINT = 'services/enhanced/check';

define( __NAMESPACE__ . '\MAIN_BANKVAL_URL', sprintf( '%s/%s', MAIN_SERVER, BANKVAL_ENDPOINT ) );
define( __NAMESPACE__ . '\BACKUP_BANKVAL_URL', sprintf( '%s/%s', BACKUP_SERVER, BANKVAL_ENDPOINT ) );

define( __NAMESPACE__ . '\MAIN_VERIFY_URL', sprintf( '%s/%s', MAIN_SERVER, VERIFY_ENDPOINT ) );
define( __NAMESPACE__ . '\BACKUP_VERIFY_URL', sprintf( '%s/%s', BACKUP_SERVER, VERIFY_ENDPOINT ) );


function get_unified_software_credentials() {
    $option = get_option( 'unified_software_credentials', '' );
    if ( '' != $option && ! is_array( $option ) ) {
        return false;
    }

    foreach ( array( 'pin', 'uname' ) as $key ) {
        if ( ! array_key_exists( $key, $option ) || '' == $option[$key] ) {
            return false;
        }
    }

    return $option;
}

function does_table_exist( $name ) {
    global $wpdb;
    return $wpdb->get_var("SHOW TABLES LIKE '${name}'") == $name;
}

function is_emailval_table( $name ) {
    global $wpdb;
    $expectedCols = array(
        'id',
        'timestamp',
        'validation_id',
        'email',
        'status',
    );

    $existingCols = $wpdb->get_col("DESC ${name}", 0);
    if ( count( $expectedCols ) != count( $existingCols ) ) {
        return false;
    }

    foreach ( $expectedCols as $col ) {
        if ( ! in_array( $col, $existingCols ) ) {
            return false;
        }
    }

    return true;
}

function create_emailval_table( $name ) {
    global $wpdb;
    $charset_collate = $wpdb->get_charset_collate();
    $sql = "CREATE TABLE ${name} (
        id BIGINT UNSIGNED AUTO_INCREMENT,
        timestamp TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
        validation_id CHAR(36),
        email VARCHAR(255) NOT NULL,
        status VARCHAR(255) NOT NULL,
        PRIMARY KEY (id),
        INDEX (timestamp),
        INDEX (validation_id),
        INDEX (status)
    ) ${charset_collate};";

    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
    dbDelta( $sql );
}

function insert_into_emailval_table( $validation_id, $email, $status, $table ) {
    global $wpdb;
    $result = $wpdb->insert(
        $table,
        array(
            'email'         => $email,
            'status'        => $status,
            'validation_id' => $validation_id
        )
    );

    return $result == 1;
}

function get_unverified_validation_id_newer_than( $table, $timestamp ) {
    global $wpdb;
    $results = $wpdb->get_results(
        $wpdb->prepare("SELECT validation_id FROM ${table} WHERE status='UNVERIFIED' AND timestamp > %s", $timestamp),
        $output = 'ARRAY_A'
    );

    return $results;
}

function update_status_by_validation_id( $table, $id, $status ) {
    global $wpdb;
    $wpdb->update( $table, array( 'status' => $status ), array( 'validation_id' => $id ) );
}

function process_verify_response( $response ) {
    if (is_wp_error($response)) {
        return false;
    }
    
    if (wp_remote_retrieve_response_code($response) != 200) {
        return false;
    }

    $respBody = wp_remote_retrieve_body($response);
    return json_decode($respBody, true);
}

function query_verify_api( $validation_id, $uname, $pin ) {
    $body = array(
        'credentials' => array(
            'pin' => $pin,
            'uname' => $uname
        ),
        'validationID'  => $validation_id
    );

    $args = array(
        'data_format' => 'body',
        'body' => json_encode($body)
    );

    $response = wp_remote_post(constant( __NAMESPACE__ . '\MAIN_VERIFY_URL' ), $args);
    $result = process_verify_response($response);

    if ( $result ) {
        return $result;
    }

    $response = wp_remote_post(constant( __NAMESPACE__ . '\BACKUP_VERIFY_URL' ), $args);
    $result = process_verify_response($response);

    if ( ! $result ) {
        return false;
    }

    return $result;
}

function process_emailval_response($response) {
    if (is_wp_error($response)) {
        return false;
    }
    
    if (wp_remote_retrieve_response_code($response) != 200) {
        return false;
    }

    $respBody = wp_remote_retrieve_body($response);
    $json = json_decode($respBody, true);

    if ( array_key_exists( 'EmailVal', $json ) ) {
        if ( is_array( $json['EmailVal'] ) ) {
            return array(
                'status'        => 1,
                'result'        => $json['EmailVal']['Result'],
                'validationID'  => $json['validationID']
            );
        } else {
            return array(
                'status'        => 0,
                'result'        => $json['EmailVal'],
                'validationID'  => $json['validationID'],
            );
        }

    }

    return array(
        'status'    => 0,
        'result'    => $json['Error']
    );
}

function emailval( $email, $uname, $pin, $verify_flag = false ) {
    $body = array(
        'credentials' => array(
            'pin' => $pin,
            'uname' => $uname
        ),
        'contact' => array(
            'emailaddress' => $email
        )
    );

    if ( $verify_flag ) {
        $body['contact']['emailverify'] = json_decode( '{}' );
    }

    $args = array(
        'data_format' => 'body',
        'body' => json_encode($body)
    );

    $response = wp_remote_post(constant( __NAMESPACE__ . '\MAIN_BANKVAL_URL' ), $args);
    $result = process_emailval_response($response);

    if ( $result ) {
        return $result;
    }

    $response = wp_remote_post(constant( __NAMESPACE__ . '\BACKUP_BANKVAL_URL' ), $args);
    $result = process_emailval_response($response);

    if ( ! $result ) {
        return false;
    }

    return $result;
}

function validate_email( $email, $pin = null, $uname = null ) {

    if ( empty( $uname ) || empty( $pin ) ) {
        $crypto = Crypto::getInstance();
        $encryptionKey = $crypto->get_encryption_key();
        if ( ! $encryptionKey ) {
            throw new Exception( 'Could not find Unified Software encryption key' );
        }
    
        $creds = get_unified_software_credentials();
    
        if ( ! $creds ) {
            throw new Exception( 'Could not find Unified Software credentials' );
        }
    
        $creds = $crypto->decrypt_arr( $creds, $encryptionKey );
        $pin = $creds['pin']; $uname = $creds['uname'];       
    }

    $option = get_option('unified_software_emailval');
    $verify_flag = array_key_exists( 'email_verification', $option );

    $response = emailval( $email, $uname, $pin, $verify_flag );

    if ( ! $response ) {
        throw new Exception( 'Could not reach Unified Software API' );
    }

    unset( $response['status'] );
    return $response;
}

?>