<?php

/**
 * Plugin Name: EmailVal - Email Address Validation & Verification
 * Description:	Check that an email address is valid and relates to a live email account.
 * Author:		Unified Software
 * Author URI:	https://www.unifiedsoftware.co.uk
 * Text Domain:	emailval
 * Version:		0.1.0
 * License:		GPL v2 or later
 * License URI:	https://www.gnu.org/licenses/gpl-2.0.html
 */

/*
    Copyright (C) 2021 Unified Software Limited
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

namespace UNIFIED_SOFTWARE\EMAILVAL;

if ( ! defined( 'WPINC' ) ) {
    die();
}

foreach( glob( plugin_dir_path( __FILE__ ) . '*.php' ) as $file ) {
    if ( ! strpos( $file, 'uninstall.php' ) ) {
        require_once $file;
    }
}

define( __NAMESPACE__ . '\UNIFIED_SOFTWARE_SETTINGS_URL', esc_url(
    add_query_arg(
        'page',
        'unified_software',
        get_admin_url() . 'admin.php'
    ))
);


function emailval_form() {
    check_ajax_referer( 'emailval_form' );
    $email = sanitize_email($_REQUEST['email']);

    $crypto = Crypto::getInstance();
    $encryptionKey = $crypto->get_encryption_key();
    if ( ! $encryptionKey ) {
        wp_send_json_error('Missing security keys', 500);
    }

    $creds = get_unified_software_credentials();
    
    if ( ! $creds ) {
        wp_send_json_error('Missing credentials', 500);
    }

    $option = get_option('unified_software_emailval');
    $should_write = $option['mysql_table'] != '';
    $verify_flag = array_key_exists( 'email_verification', $option );

    $creds = $crypto->decrypt_arr( $creds, $encryptionKey );
    $pin = $creds['pin']; $uname = $creds['uname'];
    $response = emailval($email, $uname, $pin, $verify_flag);
    $id = is_array( $response ) && array_key_exists( 'validationID', $response )
        ? $response['validationID'] : NULL;

    $status = '';
    $should_verify = array_key_exists( 'email_verification', $option );

    if ( strpos( $response['result'], 'INVALID' ) === false ) {
        if ( $should_verify ) {
            $status = 'UNVERIFIED';
        } else {
            $status = $response['result'];
        }
    } else {
        $status = $response['result'];
    }

    if ( $response ) {
        if ( $should_write ) {
            insert_into_emailval_table( $id, $email, $status, $option['mysql_table'] );
        }
        wp_send_json_success( $response, 200 );
    } else {
        if ( $should_write ) {
            insert_into_emailval_table( $id, $email, 'Server error', $option['mysql_table'] );
        }
        wp_send_json_error( $response, 500 );
    }

    die();
}

function render_emailval_form() {
	ob_start();
	include 'template/emailval-form-template.php';
	return ob_get_clean();
}

function register_emailval_scripts() {
    wp_register_script(
        'emailval-form-script',
        plugin_dir_url( __FILE__ ) . 'js/emailval-form.js',
        array( 'jquery' )
    );

    wp_add_inline_script(
        'emailval-form-script',
        'const EMAILVAL_DATA = ' . json_encode(
            array(
                'ajaxUrl' => admin_url( 'admin-ajax.php' )
            )),
        'before'
    );

    wp_enqueue_script( 'emailval-form-script' );
}

function register_emailval_styles() {
    wp_register_style(
        'emailval-form-style',
        plugin_dir_url( __FILE__ ) . 'css/emailval-form.css'
    );

    wp_enqueue_style( 'emailval-form-style' );
}

function on_emailval_activation() {
    if ( ! get_option( 'unified_software_emailval' ) ) {
        add_option( 'unified_software_emailval', array( 'polling_interval' => 'every_5_minutes' ) );
        if ( ! does_table_exist( 'emailval' ) ) {
            update_option( 'unified_software_emailval', array_merge(
                get_option( 'unified_software_emailval' ), array( 'mysql_table' => 'emailval' ) ) );
            create_emailval_table( 'emailval' );
        } else if ( is_emailval_table( 'emailval' ) ) {
            update_option( 'unified_software_emailval', array_merge(
                get_option( 'unified_software_emailval' ), array( 'mysql_table' => 'emailval' ) ) );
        }
    }
}

function on_emailval_deactivation() {
    wp_clear_scheduled_hook( 'emailval_poll_verify' );
}

function emailval_action_links( $links ) {
    $url = constant( __NAMESPACE__ . '\UNIFIED_SOFTWARE_SETTINGS_URL' );
    $settings_link = "<a href='$url'>" . __( 'Settings' ) . '</a>';

    array_unshift(
        $links,
        $settings_link
    );

    return $links;
}

function emailval_meta_links( $links, $plugin_file_name, $plugin_data, $status ) {
    if ( strpos( $plugin_file_name, basename(__FILE__) ) ) {
        $url = constant( __NAMESPACE__ . '\UNIFIED_SOFTWARE_SETTINGS_URL' );
        $support_link = 'https://www.unifiedsoftware.co.uk/contact/';

        $links[] = "<a href='$url'>" . __( 'Settings' ) . '</a>';
        $links[] = "<a href='$support_link'>" . __( 'Support' ) . '</a>';
    }
  
    return $links;
}

function emailval_custom_intervals( $schedules ) {
    $schedules['every_5_minutes'] = array(
        'interval'  => 5 * MINUTE_IN_SECONDS,
        'display'   => __( 'Every 5 minutes', 'unified-software' )
    );

    return $schedules;
}

function schedule_email_verification_polling() {
    $option = get_option( 'unified_software_emailval' );
    if (    is_array( $option )
        &&  ! wp_next_scheduled( 'emailval_poll_verify' )
        &&  array_key_exists( 'email_verification', $option )
        &&  array_key_exists( 'mysql_table', $option )
        &&  array_key_exists( 'polling_interval', $option )
    ) {
        $interval = $option['polling_interval'];
        wp_schedule_event( time(), $interval, 'emailval_poll_verify' );
    }
}

function emailval_update_verification_status() {
    $option = get_option( 'unified_software_emailval' );
    if ( ! array_key_exists( 'email_verification', $option ) || $option['mysql_table'] == '' ) return;
    $table = $option['mysql_table'];
    $crypto = Crypto::getInstance();
    $encryptionKey = $crypto->get_encryption_key();
    $creds = get_unified_software_credentials();
    $creds = $crypto->decrypt_arr( $creds, $encryptionKey );
    $pin = $creds['pin']; $uname = $creds['uname'];
    $_1_week_ago = date('Y-m-d H:i:s', strtotime('-1 week'));
    $validation_ids = get_unverified_validation_id_newer_than( $table, $_1_week_ago );
    foreach ( $validation_ids as $idx => $row ) {
        $id = $row['validation_id'];
        $response = query_verify_api( $id, $uname, $pin );
        if ( $response && array_key_exists( 'EmailVal', $response ) &&
            array_key_exists( 'Verification', $response['EmailVal'] ) &&
            $response['EmailVal']['Verification'] == 'VERIFIED' ) {
                update_status_by_validation_id( $table, $id, 'VERIFIED' );
        }
    }
}

add_shortcode( 'emailval_form', __NAMESPACE__ . '\render_emailval_form' );
add_action( 'wp_enqueue_scripts', __NAMESPACE__ . '\register_emailval_scripts' );
add_action( 'wp_enqueue_scripts', __NAMESPACE__ . '\register_emailval_styles' );
add_action( 'wp_ajax_emailval_form', __NAMESPACE__ . '\emailval_form' );
add_action( 'wp_ajax_nopriv_emailval_form', __NAMESPACE__ . '\emailval_form' );
add_action( 'admin_init', __NAMESPACE__ . '\schedule_email_verification_polling' );
add_action( 'emailval_poll_verify', __NAMESPACE__ . '\emailval_update_verification_status', 10, 0 );
add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), __NAMESPACE__ . '\emailval_action_links' );
add_filter( 'plugin_row_meta', __NAMESPACE__ . '\emailval_meta_links', 10, 4 );
add_filter( 'cron_schedules', __NAMESPACE__ . '\emailval_custom_intervals');

register_activation_hook( __FILE__, __NAMESPACE__ . '\on_emailval_activation' );
register_deactivation_hook( __FILE__, __NAMESPACE__ . '\on_emailval_deactivation' );

?>